@@ -19,14 +19,17 @@ QUIET='VERBOSE=1'
 CMAKE_BUILD_TYPE='RelWithDebInfo'
 DEBUG_COMMENT=''
 WITH_JEMALLOC=''
+DEBUG_EXTNAME=''
 WITH_SSL='/usr'
 WITH_SSL_TYPE='system'
 OPENSSL_INCLUDE=''
 OPENSSL_LIBRARY=''
 CRYPTO_LIBRARY=''
 GALERA_SSL=''
+SSL_OPT=''
 TAG=''
 #
+CMAKE_BUILD_TYPE=''
 COMMON_FLAGS=''
 #
 # Some programs that may be overriden
@@ -55,6 +58,7 @@ do
         shift
         CMAKE_BUILD_TYPE='Debug'
         BUILD_COMMENT="${BUILD_COMMENT:-}-debug"
+        DEBUG_EXTNAME='-DDEBUG_EXTNAME=OFF'
         ;;
     -v | --valgrind )
         shift
@@ -82,13 +86,13 @@ do
         if test -e "$WITH_SSL/lib/libssl.a"
         then
             OPENSSL_INCLUDE="-DOPENSSL_INCLUDE_DIR=$WITH_SSL/include"
-            OPENSSL_LIBRARY="-DOPENSSL_LIBRARIES=$WITH_SSL/lib/libssl.a"
+            OPENSSL_LIBRARY="-DOPENSSL_LIBRARY=$WITH_SSL/lib/libssl.a"
             CRYPTO_LIBRARY="-DCRYPTO_LIBRARY=$WITH_SSL/lib/libcrypto.a"
             GALERA_SSL="$WITH_SSL/lib"
         elif test -e "$WITH_SSL/lib64/libssl.a"
         then
             OPENSSL_INCLUDE="-DOPENSSL_INCLUDE_DIR=$WITH_SSL/include"
-            OPENSSL_LIBRARY="-DOPENSSL_LIBRARIES=$WITH_SSL/lib64/libssl.a"
+            OPENSSL_LIBRARY="-DOPENSSL_LIBRARY=$WITH_SSL/lib64/libssl.a"
             CRYPTO_LIBRARY="-DCRYPTO_LIBRARY=$WITH_SSL/lib64/libcrypto.a"
             GALERA_SSL="$WITH_SSL/lib64"
         else
@@ -166,7 +170,7 @@ if [[ -z ${REVISION:-} ]];then
 else 
     REVISION=$REVISION
     echo "Building with revision $REVISION"
-fi 
+fi
 
 # Extract version from the VERSION file
 source "$SOURCEDIR/VERSION"
@@ -186,8 +190,9 @@ then
 else
     REVISION=""
 fi
+REVISION="$(cd "$SOURCEDIR"; grep '^revno: ' Docs/INFO_SRC |sed -e 's/revno: //')"
 WSREP_VERSION="$(grep WSREP_INTERFACE_VERSION wsrep/wsrep_api.h | cut -d '"' -f2).$(grep 'SET(WSREP_PATCH_VERSION'  "cmake/wsrep.cmake" | cut -d '"' -f2)"
-GALERA_REVISION="$(cd "$SOURCEDIR/percona-xtradb-cluster-galera"; test -r GALERA-REVISION && cat GALERA-REVISION || bzr revno)"
+GALERA_REVISION="$(cd "$SOURCEDIR/percona-xtradb-cluster-galera"; test -r GALERA-REVISION && cat GALERA-REVISION)"
 PRODUCT_FULL="$PRODUCT-$RELEASE_TAG$WSREP_VERSION.$REVISION${BUILD_COMMENT:-}$TAG.$(uname -s).$TARGET"
 COMMENT="Percona XtraDB Cluster binary (GPL) $MYSQL_VERSION-$RELEASE_TAG$WSREP_VERSION"
 COMMENT="$COMMENT, Revision $REVISION${BUILD_COMMENT:-}"
@@ -195,6 +200,26 @@ COMMENT="$COMMENT, Revision $REVISION${BUILD_COMMENT:-}"
 # Compilation flags
 export CC=${CC:-gcc}
 export CXX=${CXX:-g++}
+
+# TokuDB cmake flags
+if test -d "$SOURCEDIR/storage/tokudb"
+then
+    CMAKE_OPTS="${CMAKE_OPTS:-} -DBUILD_TESTING=OFF -DUSE_GTAGS=OFF -DUSE_CTAGS=OFF -DUSE_ETAGS=OFF -DUSE_CSCOPE=OFF"
+    
+    if test "x$CMAKE_BUILD_TYPE" != "xDebug"
+    then
+        CMAKE_OPTS="${CMAKE_OPTS:-} -DTOKU_DEBUG_PARANOID=OFF"
+    else
+        CMAKE_OPTS="${CMAKE_OPTS:-} -DTOKU_DEBUG_PARANOID=ON"
+    fi
+
+    if [[ $CMAKE_OPTS == *WITH_VALGRIND=ON* ]]
+    then
+        CMAKE_OPTS="${CMAKE_OPTS:-} -DUSE_VALGRIND=ON"
+    fi
+fi
+
+#
 if [ -n "$(which rpm)" ]; then
   export COMMON_FLAGS=$(rpm --eval %optflags | sed -e "s|march=i386|march=i686|g")
 else
@@ -237,7 +262,7 @@ fi
             revno="$GALERA_REVISION" boost_pool=0 garb/garbd libgalera_smm.so
         else 
             scons $MAKE_JFLAG --config=force revno="$GALERA_REVISION" \
-              boost_pool=0  garb/garbd libgalera_smm.so
+                garb/garbd libgalera_smm.so
         fi
         mkdir -p "$WORKDIR/usr/local/$PRODUCT_FULL/bin" \
              "$WORKDIR/usr/local/$PRODUCT_FULL/lib"
@@ -255,9 +280,9 @@ fi
         SSL_OPT='-DWITH_SSL=system -DWITH_ZLIB=system'
     fi
     cmake . ${CMAKE_OPTS:-} -DBUILD_CONFIG=mysql_release \
-        -DCMAKE_BUILD_TYPE="$CMAKE_BUILD_TYPE" \
+        -DCMAKE_BUILD_TYPE=${CMAKE_BUILD_TYPE:-RelWithDebInfo} \
+        $DEBUG_EXTNAME \
         -DWITH_EMBEDDED_SERVER=OFF \
-        -DENABLE_DTRACE=OFF \
         -DFEATURE_SET=community \
         -DENABLE_DTRACE=OFF \
          $SSL_OPT \
@@ -268,6 +293,7 @@ fi
         -DWITH_WSREP=ON \
         -DCOMPILATION_COMMENT="$COMMENT" \
         -DWITH_PAM=ON \
+        -DWITH_INNODB_MEMCACHED=ON \
         $OPENSSL_INCLUDE $OPENSSL_LIBRARY $CRYPTO_LIBRARY
 
     make $MAKE_JFLAG $QUIET
@@ -277,6 +303,7 @@ fi
         make clean
         cmake . ${CMAKE_OPTS:-} -DBUILD_CONFIG=mysql_release \
             -DCMAKE_BUILD_TYPE=Debug \
+            $DEBUG_EXTNAME \
             -DWITH_EMBEDDED_SERVER=OFF \
             -DFEATURE_SET=community \
             -DENABLE_DTRACE=OFF \
@@ -288,27 +315,14 @@ fi
             -DWITH_WSREP=ON \
             -DCOMPILATION_COMMENT="$COMMENT" \
             -DWITH_PAM=ON \
+            -DWITH_INNODB_MEMCACHED=ON \
             $OPENSSL_INCLUDE $OPENSSL_LIBRARY $CRYPTO_LIBRARY
         make $MAKE_JFLAG $QUIET
 
         echo "Copying mysqld-debug"
-        cp -v sql/mysqld $WORKDIR/usr/local/$PRODUCT_FULL/bin/mysqld-debug
+        cp -v sql/mysqld-debug $WORKDIR/usr/local/$PRODUCT_FULL/bin/
     fi
 
-    # Build HandlerSocket
-    (
-        cd "plugin/HandlerSocket-Plugin-for-MySQL"
-        ./autogen.sh
-        CXX=${HS_CXX:-g++} ./configure --with-mysql-source="$SOURCEDIR" \
-            --with-mysql-bindir="$SOURCEDIR/scripts" \
-            --with-mysql-plugindir="/usr/local/$PRODUCT_FULL/lib/mysql/plugin" \
-            --libdir="/usr/local/$PRODUCT_FULL/lib/mysql/plugin" \
-            --prefix="/usr/local/$PRODUCT_FULL"
-        make $MAKE_JFLAG
-        make DESTDIR="$WORKDIR" install
-
-    )
-
 
     (
        echo "Packaging the test files"
@@ -323,7 +337,7 @@ fi
         cd "$JEMALLOCDIR"
 
         CFLAGS= ./autogen.sh --disable-valgrind --prefix="/usr/local/$PRODUCT_FULL/" \
-                --libdir="/usr/local/$PRODUCT_FULL/lib/mysql/"
+            --libdir="/usr/local/$PRODUCT_FULL/lib/mysql/"
         make $MAKE_JFLAG
         make DESTDIR="$WORKDIR" install_lib_shared
         strip lib/libjemalloc* || true
@@ -339,9 +353,16 @@ fi
 (
     cd "$WORKDIR/usr/local/"
 
-    $TAR czf "$WORKDIR/$PRODUCT_FULL.tar.gz" \
-        --owner=0 --group=0 "$PRODUCT_FULL/"
-    
+    find $PRODUCT_FULL ! -type d  ! \( -iname '*toku*' -o -iwholename '*/tokudb*/*' -o -iname '*tdb*.h' \) | sort > $WORKDIR/tokudb_server.list
+    $TAR --owner=0 --group=0 -czf "$WORKDIR/$PRODUCT_FULL.tar.gz" -T $WORKDIR/tokudb_server.list
+    rm -f $WORKDIR/tokudb_server.list
+
+    if test -e "$PRODUCT_FULL/lib/mysql/plugin/ha_tokudb.so"
+    then
+        TARGETTOKU=$(echo $PRODUCT_FULL | sed 's/.Linux/.TokuDB.Linux/')
+	find $PRODUCT_FULL ! -type d \( -iname '*toku*' -o -iwholename '*/tokudb*/*' -o -iname '*tdb*.h' \) > $WORKDIR/tokudb_plugin.list
+        $TAR --owner=0 --group=0 -czf "$WORKDIR/$TARGETTOKU.tar.gz" -T $WORKDIR/tokudb_plugin.list
+        rm -f $WORKDIR/tokudb_plugin.list
+    fi
 )
 
-
