@@ -19,7 +19,6 @@ mysqld_ld_preload=
 mysqld_ld_library_path=
 flush_caches=0
 numa_interleave=0
-resume_on_fail=1
 
 # Initial logging status: error log is not open, and not using syslog
 logging=init
@@ -35,6 +34,7 @@ syslog_tag_mysqld=mysqld
 syslog_tag_mysqld_safe=mysqld_safe
 
 trap '' 1 2 3 15			# we shouldn't let anyone kill us
+trap '' 13                              # not even SIGPIPE
 
 # MySQL-specific environment variable. First off, it's not really a umask,
 # it's the desired mode. Second, it follows umask(2), not umask(3) in that
@@ -251,9 +251,8 @@ wsrep_recover_position() {
   if [ -z "$rp" ]; then
     local skipped="$(grep WSREP $wr_logfile | grep 'skipping position recovery')"
     if [ -z "$skipped" ]; then
-      log_error "WSREP: Failed to recover position: 
-'`cat $wr_logfile`'"
-      ret=2
+      log_error "WSREP: Failed to recover position: " `cat $wr_logfile`;
+      ret=1
     else
       log_notice "WSREP: Position recovery skipped"
     fi
@@ -264,7 +263,7 @@ wsrep_recover_position() {
     wsrep_start_position_opt="--wsrep_start_position=$start_pos"
   fi
 
-  [ $ret -eq 0 ] && rm $wr_logfile
+  rm $wr_logfile
 
   return $ret
 }
@@ -328,12 +327,12 @@ parse_arguments() {
       --wsrep-data-home-dir=*) wsrep_data_home_dir="$val"; ;;
       --flush-caches=*) flush_caches="$val" ;;
       --numa-interleave=*) numa_interleave="$val" ;;
-      --exit-on-recover-fail) resume_on_fail=0 ;;
       --wsrep[-_]provider=*)
         if test -n "$val" && test "$val" != "none"
         then
           wsrep_restart=1
         fi
+        append_arg_to_args "$arg"
         ;;
       --help) usage ;;
 
@@ -356,19 +355,8 @@ parse_arguments() {
 # LD_LIBRARY_PATH and stripped from the lib value.
 add_mysqld_ld_preload() {
   lib_to_add="$1"
-  lib_to_add=$(readlink -f $lib_to_add)
   log_notice "Adding '$lib_to_add' to LD_PRELOAD for mysqld"
 
-  # Check if the library is in the reduced number of standard system directories
-  case "$lib_to_add" in
-    /usr/lib64/* | /usr/lib/* | ${MY_BASEDIR_VERSION}/lib/*)
-      ;;
-    *)
-      log_error "ld_preload libraries can only be loaded from system directories (/usr/lib64, /usr/lib, ${MY_BASEDIR_VERSION}/lib)"
-      exit 1
-      ;;
-  esac
-
   case "$lib_to_add" in
     *' '*)
       # Must strip path from lib, and add it to LD_LIBRARY_PATH
@@ -484,6 +472,19 @@ set_malloc_lib() {
 }
 
 
+#
+# Add jemalloc to ld_preload - needed for TokuDB
+#
+for libjemall in "/usr/lib64" "/usr/lib/x86_64-linux-gnu" "/usr/lib"
+do
+if test -f "$libjemall/libjemalloc.so.1"
+then
+  add_mysqld_ld_preload "$libjemall/libjemalloc.so.1"
+  break
+fi
+done
+
+
 #
 # First, try to find BASEDIR and ledir (where mysqld is)
 #
@@ -613,7 +614,6 @@ fi
 parse_arguments `$print_defaults $defaults --loose-verbose mysqld_safe safe_mysqld`
 parse_arguments PICK-ARGS-FROM-ARGV "$@"
 
-
 #
 # Try to find the plugin directory
 #
@@ -688,9 +688,9 @@ then
   log_notice "Logging to '$err_log'."
   logging=file
 
-  if [ ! -f "$err_log" -a ! -h "$err_log" ]; then # if error log already exists,
-    touch "$err_log"                              # we just append. otherwise,
-    chmod "$fmode" "$err_log"                     # fix the permissions here!
+  if [ ! -f "$err_log" ]; then                  # if error log already exists,
+    touch "$err_log"                            # we just append. otherwise,
+    chmod "$fmode" "$err_log"                   # fix the permissions here!
   fi
 
 else
@@ -713,7 +713,7 @@ then
     USER_OPTION="--user=$user"
   fi
   # Change the err log to the right user, if it is in use
-  if [ $want_syslog -eq 0 -a ! -h "$err_log" ]; then
+  if [ $want_syslog -eq 0 ]; then
     touch "$err_log"
     chown $user "$err_log"
   fi
@@ -733,11 +733,9 @@ safe_mysql_unix_port=${mysql_unix_port:-${MYSQL_UNIX_PORT:-@MYSQL_UNIX_ADDR@}}
 mysql_unix_port_dir=`dirname $safe_mysql_unix_port`
 if [ ! -d $mysql_unix_port_dir ]
 then
-  if [ ! -h $mysql_unix_port_dir ]; then
-    mkdir $mysql_unix_port_dir
-    chown $user $mysql_unix_port_dir
-    chmod 755 $mysql_unix_port_dir
-  fi
+  mkdir $mysql_unix_port_dir
+  chown $user $mysql_unix_port_dir
+  chmod 755 $mysql_unix_port_dir
 fi
 
 # If the user doesn't specify a binary, we assume name "mysqld"
@@ -849,11 +847,7 @@ then
       exit 1
     fi
   fi
-
-  if [ ! -h "$pid_file" ]; then
-    rm -f "$pid_file"
-  fi
-
+  rm -f "$pid_file"
   if test -f "$pid_file"
   then
     log_error "Fatal error: Can't remove the pid file:
@@ -963,6 +957,8 @@ max_fast_restarts=5
 # flag whether a usable sleep command exists
 have_sleep=1
 
+# maximum number of wsrep restarts
+max_wsrep_restarts=0
 
 # maximum number of wsrep restarts
 max_wsrep_restarts=0
@@ -975,43 +971,25 @@ fi
 
 while true
 do
-  # Some extra safety
-  if [ ! -h "$safe_mysql_unix_port" ]; then
-    rm -f "$safe_mysql_unix_port"
-  fi
-  if [ ! -h "$pid_file" ]; then
-    rm -f "$pid_file"
-  fi
+  rm -f $safe_mysql_unix_port "$pid_file"	# Some extra safety
 
   start_time=`date +%M%S`
-  
+
   # This file is checked for empty directory because 
   # a) Not having this file means the wsrep-start-position is useless - lp:1112724
   # b) Otherwise I have to check if directory is empty sans a few files like 
   # error log and others, #a is simpler.
   if [ -e $grastate_loc ];then
-    # this sets wsrep_start_position_opt
-    wsrep_recover_position "$cmd"
+  # this sets wsrep_start_position_opt
+  wsrep_recover_position "$cmd"
   else 
     log_notice "Skipping wsrep-recover for empty datadir: ${DATADIR}"
     log_notice "Assigning 00000000-0000-0000-0000-000000000000:-1 to wsrep_start_position"
     wsrep_start_position_opt="--wsrep_start_position='00000000-0000-0000-0000-000000000000:-1'"
   fi
 
-  retcode=$?
 
-  if test $retcode -eq 2;then
-      if  test $resume_on_fail -eq 1;then
-        log_notice "wsrep-recovery has failed to recover, continuing with startup"
-        wsrep_start_position_opt=""
-      else 
-        log_error " --exit-on-recover-fail is provided, bailing out"
-        exit 2
-      fi
-  elif test $retcode -ne 0;then
-      log_error "Unknown error: $retcode"
-      exit $retcode
-  fi
+  [ $? -ne 0 ] && exit 1 #
 
   [ -n "$wsrep_urls" ] && url=`wsrep_pick_url $wsrep_urls` # check connect address
 
@@ -1022,7 +1000,7 @@ do
     eval_log_error "$cmd $wsrep_start_position_opt --wsrep_cluster_address=$url $nohup_redir"
   fi
 
-  if [ $want_syslog -eq 0 -a ! -f "$err_log" -a ! -h "$err_log" ]; then
+  if [ $want_syslog -eq 0 -a ! -f "$err_log" ]; then
     touch "$err_log"                    # hypothetical: log was renamed but not
     chown $user "$err_log"              # flushed yet. we'd recreate it with
     chmod "$fmode" "$err_log"           # wrong owner next time we log, so set
