@@ -29,8 +29,6 @@ ekeyfile=""
 encrypt=0
 nproc=1
 ecode=0
-ssyslog=""
-ssystag=""
 XTRABACKUP_PID=""
 SST_PORT=""
 REMOTEIP=""
@@ -42,15 +40,15 @@ progress=""
 ttime=0
 totime=0
 lsn=""
+incremental=0
 ecmd=""
 rlimit=""
 # Initially
 stagemsg="${WSREP_SST_OPT_ROLE}"
 cpat=""
-speciald=1
+speciald=0
 ib_home_dir=""
 ib_log_dir=""
-ib_undo_dir=""
 
 sfmt="tar"
 strmcmd=""
@@ -259,16 +257,6 @@ get_footprint()
 
 adjust_progress()
 {
-
-    if [[ ! -x `which pv` ]];then 
-        wsrep_log_error "pv not found in path: $PATH"
-        wsrep_log_error "Disabling all progress/rate-limiting"
-        pcmd=""
-        rlimit=""
-        progress=""
-        return
-    fi
-
     if [[ -n $progress && $progress != '1' ]];then 
         if [[ -e $progress ]];then 
             pcmd+=" 2>>$progress"
@@ -298,7 +286,8 @@ read_cnf()
     progress=$(parse_cnf sst progress "")
     rebuild=$(parse_cnf sst rebuild 0)
     ttime=$(parse_cnf sst time 0)
-    cpat=$(parse_cnf sst cpat '.*galera\.cache$\|.*sst_in_progress$\|.*\.sst$\|.*gvwstate\.dat$\|.*grastate\.dat$\|.*\.err$\|.*\.log$\|.*RPM_UPGRADE_MARKER$\|.*RPM_UPGRADE_HISTORY$')
+    cpat=$(parse_cnf sst cpat '.*galera\.cache$\|.*sst_in_progress$\|.*grastate\.dat$\|.*\.err$\|.*\.log$\|.*RPM_UPGRADE_MARKER$\|.*RPM_UPGRADE_HISTORY$')
+    incremental=$(parse_cnf sst incremental 0)
     ealgo=$(parse_cnf xtrabackup encrypt "")
     ekey=$(parse_cnf xtrabackup encrypt-key "")
     ekeyfile=$(parse_cnf xtrabackup encrypt-key-file "")
@@ -319,20 +308,6 @@ read_cnf()
     iapts=$(parse_cnf sst inno-apply-opts "")
     impts=$(parse_cnf sst inno-move-opts "")
     stimeout=$(parse_cnf sst sst-initial-timeout 100)
-    ssyslog=$(parse_cnf sst sst-syslog 0)
-    ssystag=$(parse_cnf mysqld_safe syslog-tag "${SST_SYSLOG_TAG:-}")
-    ssystag+="-"
-
-    if [[ $speciald -eq 0 ]];then 
-        wsrep_log_error "sst-special-dirs equal to 0 is not supported, falling back to 1"
-        speciald=1
-    fi 
-
-    if [[ $ssyslog -ne -1 ]];then 
-        if my_print_defaults -c $WSREP_SST_OPT_CONF mysqld_safe | tr '_' '-' | grep -q -- "--syslog";then 
-            ssyslog=1
-        fi
-    fi
 }
 
 get_stream()
@@ -376,7 +351,8 @@ cleanup_joiner()
     local estatus=$?
     if [[ $estatus -ne 0 ]];then 
         wsrep_log_error "Cleanup after exit with status:$estatus"
-    elif [ "${WSREP_SST_OPT_ROLE}" = "joiner" ];then
+    fi
+    if [ "${WSREP_SST_OPT_ROLE}" = "joiner" ];then
         wsrep_log_info "Removing the sst_in_progress file"
         wsrep_cleanup_progress_file
     fi
@@ -387,23 +363,6 @@ cleanup_joiner()
     if [[ -n ${STATDIR:-} ]];then 
        [[ -d $STATDIR ]] && rm -rf $STATDIR
     fi
-
-    # Final cleanup 
-    pgid=$(ps -o pgid= $$ | grep -o '[0-9]*')
-
-    # This means no setsid done in mysqld.
-    # We don't want to kill mysqld here otherwise.
-    if [[ $$ -eq $pgid ]];then
-
-        # This means a signal was delivered to the process.
-        # So, more cleanup. 
-        if [[ $estatus -ge 128 ]];then 
-            kill -KILL -$$ || true
-        fi
-
-    fi
-
-    exit $estatus
 }
 
 check_pid()
@@ -444,23 +403,6 @@ cleanup_donor()
     if [[ -n $itmpdir ]];then 
        [[ -d $itmpdir ]] &&  rm -rf $itmpdir || true
     fi
-
-    # Final cleanup 
-    pgid=$(ps -o pgid= $$ | grep -o '[0-9]*')
-
-    # This means no setsid done in mysqld.
-    # We don't want to kill mysqld here otherwise.
-    if [[ $$ -eq $pgid ]];then
-
-        # This means a signal was delivered to the process.
-        # So, more cleanup. 
-        if [[ $estatus -ge 128 ]];then 
-            kill -KILL -$$ || true
-        fi
-
-    fi
-
-    exit $estatus
 }
 
 kill_xtrabackup()
@@ -494,7 +436,11 @@ wait_for_listen()
         ss -p state listening "( sport = :$PORT )" | grep -qE 'socat|nc' && break
         sleep 0.2
     done
-    echo "ready ${ADDR}/${MODULE}"
+    if [[ $incremental -eq 1 ]];then 
+        echo "ready ${ADDR}/${MODULE}/$lsn"
+    else 
+        echo "ready ${ADDR}/${MODULE}"
+    fi
 }
 
 check_extra()
@@ -536,7 +482,7 @@ recv_joiner()
         if timeout --help | grep -q -- '-k';then 
             ltcmd="timeout -k $(( tmt+10 )) $tmt $tcmd"
         else 
-            ltcmd="timeout -s9 $tmt $tcmd"
+            ltcmd="timeout $tmt $tcmd"
         fi
         timeit "$msg" "$ltcmd | $strmcmd; RC=( "\${PIPESTATUS[@]}" )"
     else 
@@ -605,47 +551,18 @@ fi
 
 read_cnf
 setup_ports
+get_stream
+get_transfer
 
-if ${INNOBACKUPEX_BIN} /tmp --help 2>/dev/null | grep -q -- '--version-check'; then 
+if ${INNOBACKUPEX_BIN} /tmp --help  | grep -q -- '--version-check'; then 
     disver="--no-version-check"
 fi
 
 
 INNOEXTRA=""
-
-if [[ $ssyslog -eq 1 ]];then 
-
-    if [[ ! -x `which logger` ]];then 
-        wsrep_log_error "logger not in path: $PATH. Ignoring"
-    else
-
-        wsrep_log_info "Logging all stderr of SST/Innobackupex to syslog"
-
-        exec 2> >(logger -p daemon.err -t ${ssystag}wsrep-sst-$WSREP_SST_OPT_ROLE)
-
-        wsrep_log_error()
-        {
-            logger  -p daemon.err -t ${ssystag}wsrep-sst-$WSREP_SST_OPT_ROLE "$@" 
-        }
-
-        wsrep_log_info()
-        {
-            logger  -p daemon.info -t ${ssystag}wsrep-sst-$WSREP_SST_OPT_ROLE "$@" 
-        }
-
-        INNOAPPLY="${INNOBACKUPEX_BIN} $disver $iapts --apply-log \$rebuildcmd \${DATA} 2>&1  | logger -p daemon.err -t ${ssystag}innobackupex-apply "
-        INNOMOVE="${INNOBACKUPEX_BIN} --defaults-file=${WSREP_SST_OPT_CONF} $disver $impts  --move-back --force-non-empty-directories \${DATA} 2>&1 | logger -p daemon.err -t ${ssystag}innobackupex-move "
-        INNOBACKUP="${INNOBACKUPEX_BIN} --defaults-file=${WSREP_SST_OPT_CONF} $disver $iopts \$tmpopts \$INNOEXTRA --galera-info --stream=\$sfmt \$itmpdir 2> >(logger -p daemon.err -t ${ssystag}innobackupex-backup)"
-    fi
-
-else 
-    INNOAPPLY="${INNOBACKUPEX_BIN} $disver $iapts --apply-log \$rebuildcmd \${DATA} &>\${DATA}/innobackup.prepare.log"
-    INNOMOVE="${INNOBACKUPEX_BIN} --defaults-file=${WSREP_SST_OPT_CONF} $disver $impts  --move-back --force-non-empty-directories \${DATA} &>\${DATA}/innobackup.move.log"
-    INNOBACKUP="${INNOBACKUPEX_BIN} --defaults-file=${WSREP_SST_OPT_CONF} $disver $iopts \$tmpopts \$INNOEXTRA --galera-info --stream=\$sfmt \$itmpdir 2>\${DATA}/innobackup.backup.log"
-fi
-
-get_stream
-get_transfer
+INNOAPPLY="${INNOBACKUPEX_BIN} $disver $iapts --apply-log \$rebuildcmd \${DATA} &>\${DATA}/innobackup.prepare.log"
+INNOMOVE="${INNOBACKUPEX_BIN} --defaults-file=${WSREP_SST_OPT_CONF} $disver $impts  --move-back --force-non-empty-directories \${DATA} &>\${DATA}/innobackup.move.log"
+INNOBACKUP="${INNOBACKUPEX_BIN} --defaults-file=${WSREP_SST_OPT_CONF} $disver $iopts \$tmpopts \$INNOEXTRA --galera-info --stream=\$sfmt \$itmpdir 2>\${DATA}/innobackup.backup.log"
 
 if [ "$WSREP_SST_OPT_ROLE" = "donor" ]
 then
@@ -683,6 +600,9 @@ then
             fi
         fi
 
+        if [[ -n $lsn ]];then 
+                INNOEXTRA+=" --incremental --incremental-lsn=$lsn "
+        fi
 
         check_extra
 
@@ -770,12 +690,25 @@ then
     [[ -e $SST_PROGRESS_FILE ]] && wsrep_log_info "Stale sst_in_progress file: $SST_PROGRESS_FILE"
     [[ -n $SST_PROGRESS_FILE ]] && touch $SST_PROGRESS_FILE
 
-    ib_home_dir=$(parse_cnf mysqld innodb-data-home-dir "")
-    ib_log_dir=$(parse_cnf mysqld innodb-log-group-home-dir "")
-    ib_undo_dir=$(parse_cnf mysqld innodb-undo-directory "")
+    if [[ $speciald -eq 1 ]];then 
+        ib_home_dir=$(parse_cnf mysqld innodb-data-home-dir "")
+        ib_log_dir=$(parse_cnf mysqld innodb-log-group-home-dir "")
+        if [[ -z $ib_home_dir && -z $ib_log_dir ]];then 
+            speciald=0
+        fi
+    fi
 
     stagemsg="Joiner-Recv"
 
+    if [[ ! -e ${DATA}/ibdata1 ]];then 
+        incremental=0
+    fi
+
+    if [[ $incremental -eq 1 ]];then 
+        wsrep_log_info "Incremental SST enabled: NOT SUPPORTED yet"
+        lsn=$(grep to_lsn xtrabackup_checkpoints | cut -d= -f2 | tr -d ' ')
+        wsrep_log_info "Recovered LSN: $lsn"
+    fi
 
     sencrypted=1
     nthreads=1
@@ -804,6 +737,11 @@ then
         tcmd+=" | $pcmd"
     fi
 
+    if [[ $incremental -eq 1 ]];then 
+        BDATA=$DATA
+        DATA=$(mktemp -d)
+        MAGIC_FILE="${DATA}/${INFO_FILE}"
+    fi
 
     get_keys
     if [[ $encrypt -eq 1 && $sencrypted -eq 1 ]];then
@@ -820,13 +758,6 @@ then
     MAGIC_FILE="${STATDIR}/${INFO_FILE}"
     recv_joiner $STATDIR  "${stagemsg}-gtid" $stimeout
 
-    if [[ -d ${DATA}/.sst ]];then 
-        wsrep_log_info "WARNING: Stale temporary SST directory: ${DATA}/.sst from previous state transfer"
-    fi
-    mkdir -p ${DATA}/.sst
-    (recv_joiner $DATA/.sst "${stagemsg}-SST" 0) &
-    jpid=$!
-
     if ! ps -p ${WSREP_SST_OPT_PARENT} &>/dev/null
     then
         wsrep_log_error "Parent mysqld process (PID:${WSREP_SST_OPT_PARENT}) terminated unexpectedly." 
@@ -837,10 +768,18 @@ then
     then
         wsrep_log_info "Proceeding with SST"
 
+        if [[ $speciald -eq 1 && -d ${DATA}/.sst ]];then 
+            wsrep_log_info "WARNING: Stale temporary SST directory: ${DATA}/.sst from previous SST"
+        fi
 
-        wsrep_log_info "Cleaning the existing datadir and innodb-data/log directories"
-        find $ib_home_dir $ib_log_dir $ib_undo_dir $DATA -mindepth 1  -regex $cpat  -prune  -o -exec rm -rfv {} 1>&2 \+
-
+        if [[ $incremental -ne 1 ]];then 
+            if [[ $speciald -eq 1 ]];then 
+                wsrep_log_info "Cleaning the existing datadir and innodb-data/log directories"
+                find $ib_home_dir $ib_log_dir $DATA -mindepth 1  -regex $cpat  -prune  -o -exec rm -rfv {} 1>&2 \+
+            else 
+                wsrep_log_info "Cleaning the existing datadir"
+                find $DATA -mindepth 1  -regex $cpat  -prune  -o -exec rm -rfv {} 1>&2 \+
+            fi
             tempdir=$(parse_cnf mysqld log-bin "")
             if [[ -n ${tempdir:-} ]];then
                 binlog_dir=$(dirname $tempdir)
@@ -853,23 +792,24 @@ then
                 fi
             fi
 
+        else
+            wsrep_log_info "Removing existing ib_logfile files"
+            rm -f ${BDATA}/ib_logfile*
+        fi
 
 
-        TDATA=${DATA}
-        DATA="${DATA}/.sst"
+        if [[ $speciald -eq 1 ]];then 
+            mkdir -p ${DATA}/.sst
+            TDATA=${DATA}
+            DATA="${DATA}/.sst"
+        fi
 
 
         MAGIC_FILE="${DATA}/${INFO_FILE}"
-        wsrep_log_info "Waiting for SST streaming to complete!"
-        wait $jpid
+        recv_joiner $DATA "${stagemsg}-SST" 0
 
         get_proc
 
-        if [[ ! -s ${DATA}/xtrabackup_checkpoints ]];then 
-            wsrep_log_error "xtrabackup_checkpoints missing, failed innobackupex/SST on donor"
-            exit 2
-        fi
-
         # Rebuild indexes for compact backups
         if grep -q 'compact = 1' ${DATA}/xtrabackup_checkpoints;then 
             wsrep_log_info "Index compaction detected"
@@ -925,6 +865,28 @@ then
         fi
 
 
+        if  [[ ! -z $WSREP_SST_OPT_BINLOG ]];then
+
+            BINLOG_DIRNAME=$(dirname $WSREP_SST_OPT_BINLOG)
+            BINLOG_FILENAME=$(basename $WSREP_SST_OPT_BINLOG)
+
+            # To avoid comparing data directory and BINLOG_DIRNAME 
+            mv $DATA/${BINLOG_FILENAME}.* $BINLOG_DIRNAME/ 2>/dev/null || true
+
+            pushd $BINLOG_DIRNAME &>/dev/null
+            for bfiles in $(ls -1 ${BINLOG_FILENAME}.*);do
+                echo ${BINLOG_DIRNAME}/${bfiles} >> ${BINLOG_FILENAME}.index
+            done
+            popd &> /dev/null
+
+        fi
+
+        if [[ $incremental -eq 1 ]];then 
+            # Added --ibbackup=xtrabackup_55 because it fails otherwise citing connection issues.
+            INNOAPPLY="${INNOBACKUPEX_BIN} $disver --defaults-file=${WSREP_SST_OPT_CONF} \
+                --ibbackup=xtrabackup_56 --apply-log $rebuildcmd --redo-only $BDATA --incremental-dir=${DATA} &>>${BDATA}/innobackup.prepare.log"
+        fi
+
         wsrep_log_info "Preparing the backup at ${DATA}"
         timeit "Xtrabackup prepare stage" "$INNOAPPLY"
 
@@ -934,25 +896,30 @@ then
             exit 22
         fi
 
-        MAGIC_FILE="${TDATA}/${INFO_FILE}"
-        set +e
-        rm $TDATA/innobackup.prepare.log $TDATA/innobackup.move.log
-        set -e
-        wsrep_log_info "Moving the backup to ${TDATA}"
-        timeit "Xtrabackup move stage" "$INNOMOVE"
-        if [[ $? -eq 0 ]];then 
-            wsrep_log_info "Move successful, removing ${DATA}"
-            rm -rf $DATA
-            DATA=${TDATA}
-        else 
-            wsrep_log_error "Move failed, keeping ${DATA} for further diagnosis"
-            wsrep_log_error "Check ${DATA}/innobackup.move.log for details"
+        if [[ $speciald -eq 1 ]];then 
+            MAGIC_FILE="${TDATA}/${INFO_FILE}"
+            set +e
+            rm $TDATA/innobackup.prepare.log $TDATA/innobackup.move.log
+            set -e
+            wsrep_log_info "Moving the backup to ${TDATA}"
+            timeit "Xtrabackup move stage" "$INNOMOVE"
+            if [[ $? -eq 0 ]];then 
+                wsrep_log_info "Move successful, removing ${DATA}"
+                rm -rf $DATA
+                DATA=${TDATA}
+            else 
+                wsrep_log_error "Move failed, keeping ${DATA} for further diagnosis"
+                wsrep_log_error "Check ${DATA}/innobackup.move.log for details"
+            fi
+        fi
+
+        if [[ $incremental -eq 1 ]];then 
+            wsrep_log_info "Cleaning up ${DATA} after incremental SST"
+            [[ -d ${DATA} ]] && rm -rf ${DATA}
+            DATA=$BDATA
         fi
 
     else 
-        # || true if it has already exited
-        kill $jpid || true
-        rm -rf $DATA/.sst
         wsrep_log_info "${IST_FILE} received from donor: Running IST"
     fi
 
@@ -960,7 +927,6 @@ then
         wsrep_log_error "SST magic file ${MAGIC_FILE} not found/readable"
         exit 2
     fi
-    wsrep_log_info "Galera co-ords from recovery: $(cat ${MAGIC_FILE})"
     cat "${MAGIC_FILE}" # output UUID:seqno
     wsrep_log_info "Total time on joiner: $totime seconds"
 fi
