@@ -10,21 +10,27 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+   along with this program; if not, write to the Free Software Foundation,
+   51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA */
 
 
-#ifdef USE_PRAGMA_IMPLEMENTATION
-#pragma implementation                         // gcc: Class implementation
-#endif
-
 #include "sql_priv.h"
 #include "transaction.h"
 #include "rpl_handler.h"
 #include "debug_sync.h"         // DEBUG_SYNC
+#include "sql_acl.h"            // SUPER_ACL
 
-/* Conditions under which the transaction state must not change. */
-static bool trans_check(THD *thd)
+/**
+  Check if we have a condition where the transaction state must
+  not be changed (committed or rolled back). Currently we check
+  that we are not executing a stored program and that we don't
+  have an active XA transaction.
+
+  @return TRUE if the commit/rollback cannot be executed,
+          FALSE otherwise.
+*/
+
+bool trans_check_state(THD *thd)
 {
   enum xa_states xa_state= thd->transaction.xid_state.xa_state;
   DBUG_ENTER("trans_check");
@@ -126,7 +132,7 @@ bool trans_begin(THD *thd, uint flags)
   int res= FALSE;
   DBUG_ENTER("trans_begin");
 
-  if (trans_check(thd))
+  if (trans_check_state(thd))
     DBUG_RETURN(TRUE);
 
   thd->locked_tables_list.unlock_locked_tables(thd);
@@ -140,15 +146,17 @@ bool trans_begin(THD *thd, uint flags)
 #ifdef WITH_WSREP
     wsrep_register_hton(thd, TRUE);
 #endif /* WITH_WSREP */
-    thd->server_status&= ~SERVER_STATUS_IN_TRANS;
-    res= test(ha_commit_trans(thd, TRUE));
+    thd->server_status&=
+      ~(SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
+    DBUG_PRINT("info", ("clearing SERVER_STATUS_IN_TRANS"));
+    res= MY_TEST(ha_commit_trans(thd, TRUE));
 #ifdef WITH_WSREP
     wsrep_post_commit(thd, TRUE);
 #endif /* WITH_WSREP */
   }
 
-  thd->variables.option_bits&= ~(OPTION_BEGIN | OPTION_KEEP_LOG);
-  thd->transaction.all.modified_non_trans_table= FALSE;
+  thd->variables.option_bits&= ~OPTION_BEGIN;
+  thd->transaction.all.reset_unsafe_rollback_flags();
 
   if (res)
     DBUG_RETURN(TRUE);
@@ -159,6 +167,29 @@ bool trans_begin(THD *thd, uint flags)
   */
   thd->mdl_context.release_transactional_locks();
 
+  // The RO/RW options are mutually exclusive.
+  DBUG_ASSERT(!((flags & MYSQL_START_TRANS_OPT_READ_ONLY) &&
+                (flags & MYSQL_START_TRANS_OPT_READ_WRITE)));
+  if (flags & MYSQL_START_TRANS_OPT_READ_ONLY)
+    thd->tx_read_only= true;
+  else if (flags & MYSQL_START_TRANS_OPT_READ_WRITE)
+  {
+    /*
+      Explicitly starting a RW transaction when the server is in
+      read-only mode, is not allowed unless the user has SUPER priv.
+      Implicitly starting a RW transaction is allowed for backward
+      compatibility.
+    */
+    const bool user_is_super=
+      MY_TEST(thd->security_ctx->master_access & SUPER_ACL);
+    if (opt_readonly && !user_is_super)
+    {
+      my_error(ER_OPTION_PREVENTS_STATEMENT, MYF(0), "--read-only");
+      DBUG_RETURN(true);
+    }
+    thd->tx_read_only= false;
+  }
+
 #ifdef WITH_WSREP
   thd->wsrep_PA_safe= true;
   if (WSREP_CLIENT(thd) && wsrep_sync_wait(thd))
@@ -167,11 +198,15 @@ bool trans_begin(THD *thd, uint flags)
 
   thd->variables.option_bits|= OPTION_BEGIN;
   thd->server_status|= SERVER_STATUS_IN_TRANS;
+  if (thd->tx_read_only)
+    thd->server_status|= SERVER_STATUS_IN_TRANS_READONLY;
+  DBUG_PRINT("info", ("setting SERVER_STATUS_IN_TRANS"));
 
+  /* ha_start_consistent_snapshot() relies on OPTION_BEGIN flag set. */
   if (flags & MYSQL_START_TRANS_OPT_WITH_CONS_SNAPSHOT)
     res= ha_start_consistent_snapshot(thd);
 
-  DBUG_RETURN(test(res));
+  DBUG_RETURN(MY_TEST(res));
 }
 
 
@@ -189,30 +224,34 @@ bool trans_commit(THD *thd)
   int res;
   DBUG_ENTER("trans_commit");
 
-  if (trans_check(thd))
+#ifndef DBUG_OFF
+  char buf1[256], buf2[256];
+  DBUG_PRINT("enter", ("stmt.ha_list: %s, all.ha_list: %s",
+                       ha_list_names(thd->transaction.stmt.ha_list, buf1),
+                       ha_list_names(thd->transaction.all.ha_list, buf2)));
+
+  thd->transaction.stmt.dbug_unsafe_rollback_flags("stmt");
+  thd->transaction.all.dbug_unsafe_rollback_flags("all");
+#endif
+
+  if (trans_check_state(thd))
     DBUG_RETURN(TRUE);
 
 #ifdef WITH_WSREP
   wsrep_register_hton(thd, TRUE);
 #endif /* WITH_WSREP */
-  thd->server_status&= ~SERVER_STATUS_IN_TRANS;
+  thd->server_status&=
+    ~(SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
+  DBUG_PRINT("info", ("clearing SERVER_STATUS_IN_TRANS"));
   res= ha_commit_trans(thd, TRUE);
+  thd->variables.option_bits&= ~OPTION_BEGIN;
+  thd->transaction.all.reset_unsafe_rollback_flags();
 #ifdef WITH_WSREP
   wsrep_post_commit(thd, TRUE);
 #endif /* WITH_WSREP */
-  if (res)
-    /*
-      if res is non-zero, then ha_commit_trans has rolled back the
-      transaction, so the hooks for rollback will be called.
-    */
-    RUN_HOOK(transaction, after_rollback, (thd, FALSE));
-  else
-    RUN_HOOK(transaction, after_commit, (thd, FALSE));
-  thd->variables.option_bits&= ~(OPTION_BEGIN | OPTION_KEEP_LOG);
-  thd->transaction.all.modified_non_trans_table= FALSE;
   thd->lex->start_transaction_opt= 0;
 
-  DBUG_RETURN(test(res));
+  DBUG_RETURN(MY_TEST(res));
 }
 
 
@@ -232,8 +271,24 @@ bool trans_commit_implicit(THD *thd)
   bool res= FALSE;
   DBUG_ENTER("trans_commit_implicit");
 
-  if (trans_check(thd))
-    DBUG_RETURN(TRUE);
+#ifndef DBUG_OFF
+  char buf1[256], buf2[256];
+  DBUG_PRINT("enter", ("stmt.ha_list: %s, all.ha_list: %s",
+                       ha_list_names(thd->transaction.stmt.ha_list, buf1),
+                       ha_list_names(thd->transaction.all.ha_list, buf2)));
+
+  thd->transaction.stmt.dbug_unsafe_rollback_flags("stmt");
+  thd->transaction.all.dbug_unsafe_rollback_flags("all");
+#endif
+
+  /*
+    Ensure that trans_check_state() was called before trans_commit_implicit()
+    by asserting that conditions that are checked in the former function are
+    true.
+  */
+  DBUG_ASSERT(thd->transaction.stmt.is_empty() &&
+              !thd->in_sub_stmt &&
+              thd->transaction.xid_state.xa_state == XA_NOTR);
 
   if (thd->in_multi_stmt_transaction_mode() ||
       (thd->variables.option_bits & OPTION_TABLE_LOCK))
@@ -244,23 +299,28 @@ bool trans_commit_implicit(THD *thd)
 #ifdef WITH_WSREP
     wsrep_register_hton(thd, TRUE);
 #endif /* WITH_WSREP */
-    thd->server_status&= ~SERVER_STATUS_IN_TRANS;
-    res= test(ha_commit_trans(thd, TRUE));
+     thd->server_status&=
+      ~(SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
+    DBUG_PRINT("info", ("clearing SERVER_STATUS_IN_TRANS"));
+    res= MY_TEST(ha_commit_trans(thd, TRUE));
 #ifdef WITH_WSREP
     wsrep_post_commit(thd, TRUE);
 #endif /* WITH_WSREP */
   }
+  else if (tc_log)
+    tc_log->commit(thd, true);
 
-  thd->variables.option_bits&= ~(OPTION_BEGIN | OPTION_KEEP_LOG);
-  thd->transaction.all.modified_non_trans_table= FALSE;
+  thd->variables.option_bits&= ~OPTION_BEGIN;
+  thd->transaction.all.reset_unsafe_rollback_flags();
 
   /*
     Upon implicit commit, reset the current transaction
-    isolation level. We do not care about
+    isolation level and access mode. We do not care about
     @@session.completion_type since it's documented
     to not have any effect on implicit commit.
   */
   thd->tx_isolation= (enum_tx_isolation) thd->variables.tx_isolation;
+  thd->tx_read_only= thd->variables.tx_read_only;
 
   DBUG_RETURN(res);
 }
@@ -278,24 +338,36 @@ bool trans_commit_implicit(THD *thd)
 bool trans_rollback(THD *thd)
 {
   int res;
-  DBUG_ENTER("trans_rollback"); 
+  DBUG_ENTER("trans_rollback");
 #ifdef WITH_WSREP
   thd->wsrep_PA_safe= true;
 #endif /* WITH_WSREP */
- if (trans_check(thd))
+
+#ifndef DBUG_OFF
+  char buf1[256], buf2[256];
+  DBUG_PRINT("enter", ("stmt.ha_list: %s, all.ha_list: %s",
+                       ha_list_names(thd->transaction.stmt.ha_list, buf1),
+                       ha_list_names(thd->transaction.all.ha_list, buf2)));
+
+  thd->transaction.stmt.dbug_unsafe_rollback_flags("stmt");
+  thd->transaction.all.dbug_unsafe_rollback_flags("all");
+#endif
+
+  if (trans_check_state(thd))
     DBUG_RETURN(TRUE);
 
 #ifdef WITH_WSREP
   wsrep_register_hton(thd, TRUE);
 #endif /* WITH_WSREP */
-  thd->server_status&= ~SERVER_STATUS_IN_TRANS;
+  thd->server_status&=
+    ~(SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
+  DBUG_PRINT("info", ("clearing SERVER_STATUS_IN_TRANS"));
   res= ha_rollback_trans(thd, TRUE);
-  RUN_HOOK(transaction, after_rollback, (thd, FALSE));
-  thd->variables.option_bits&= ~(OPTION_BEGIN | OPTION_KEEP_LOG);
-  thd->transaction.all.modified_non_trans_table= FALSE;
+  thd->variables.option_bits&= ~OPTION_BEGIN;
+  thd->transaction.all.reset_unsafe_rollback_flags();
   thd->lex->start_transaction_opt= 0;
 
-  DBUG_RETURN(test(res));
+  DBUG_RETURN(MY_TEST(res));
 }
 
 
@@ -330,7 +402,8 @@ bool trans_rollback_implicit(THD *thd)
 #ifdef WITH_WSREP
   wsrep_register_hton(thd, true);
 #endif /* WITH_WSREP */
-  thd->server_status&= ~SERVER_STATUS_IN_TRANS;
+  thd->server_status&=
+    ~(SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
   DBUG_PRINT("info", ("clearing SERVER_STATUS_IN_TRANS"));
   res= ha_rollback_trans(thd, true);
   /*
@@ -338,13 +411,12 @@ bool trans_rollback_implicit(THD *thd)
     of new transacton in @@autocommit=1 mode. This is necessary to
     preserve backward compatibility.
   */
-  thd->variables.option_bits&= ~(OPTION_KEEP_LOG);
-  thd->transaction.all.modified_non_trans_table= false;
+  thd->transaction.all.reset_unsafe_rollback_flags();
 
   /* Rollback should clear transaction_rollback_request flag. */
   DBUG_ASSERT(! thd->transaction_rollback_request);
 
-  DBUG_RETURN(test(res));
+  DBUG_RETURN(MY_TEST(res));
 }
 
 
@@ -366,6 +438,13 @@ bool trans_rollback_implicit(THD *thd)
 bool trans_commit_stmt(THD *thd)
 {
   DBUG_ENTER("trans_commit_stmt");
+#ifndef DBUG_OFF
+  char buf1[256], buf2[256];
+  DBUG_PRINT("enter", ("stmt.ha_list: %s, all.ha_list: %s",
+                       ha_list_names(thd->transaction.stmt.ha_list, buf1),
+                       ha_list_names(thd->transaction.all.ha_list, buf2)));
+#endif
+
   int res= FALSE;
   /*
     We currently don't invoke commit/rollback at end of
@@ -375,6 +454,17 @@ bool trans_commit_stmt(THD *thd)
   */
   DBUG_ASSERT(! thd->in_sub_stmt);
 
+#ifndef DBUG_OFF
+  DBUG_PRINT("enter", ("stmt.ha_list: %s, all.ha_list: %s",
+                       ha_list_names(thd->transaction.stmt.ha_list, buf1),
+                       ha_list_names(thd->transaction.all.ha_list, buf2)));
+
+  thd->transaction.stmt.dbug_unsafe_rollback_flags("stmt");
+  thd->transaction.all.dbug_unsafe_rollback_flags("all");
+#endif
+
+  thd->transaction.merge_unsafe_rollback_flags();
+
   if (thd->transaction.stmt.ha_list)
   {
 #ifdef WITH_WSREP
@@ -386,24 +476,18 @@ bool trans_commit_stmt(THD *thd)
     {
 #endif /* WITH_WSREP */
       thd->tx_isolation= (enum_tx_isolation) thd->variables.tx_isolation;
+      thd->tx_read_only= thd->variables.tx_read_only;
 #ifdef WITH_WSREP
       wsrep_post_commit(thd, FALSE);
     }
 #endif /* WITH_WSREP */
   }
-
-  if (res)
-    /*
-      if res is non-zero, then ha_commit_trans has rolled back the
-      transaction, so the hooks for rollback will be called.
-    */
-    RUN_HOOK(transaction, after_rollback, (thd, FALSE));
-  else
-    RUN_HOOK(transaction, after_commit, (thd, FALSE));
+  else if (tc_log)
+    tc_log->commit(thd, false);
 
   thd->transaction.stmt.reset();
 
-  DBUG_RETURN(test(res));
+  DBUG_RETURN(MY_TEST(res));
 }
 
 
@@ -427,6 +511,18 @@ bool trans_rollback_stmt(THD *thd)
   */
   DBUG_ASSERT(! thd->in_sub_stmt);
 
+#ifndef DBUG_OFF
+  char buf1[256], buf2[256];
+  DBUG_PRINT("enter", ("stmt.ha_list: %s, all.ha_list: %s",
+                       ha_list_names(thd->transaction.stmt.ha_list, buf1),
+                       ha_list_names(thd->transaction.all.ha_list, buf2)));
+
+  thd->transaction.stmt.dbug_unsafe_rollback_flags("stmt");
+  thd->transaction.all.dbug_unsafe_rollback_flags("all");
+#endif
+
+  thd->transaction.merge_unsafe_rollback_flags();
+
   if (thd->transaction.stmt.ha_list)
   {
 #ifdef WITH_WSREP
@@ -434,10 +530,13 @@ bool trans_rollback_stmt(THD *thd)
 #endif /* WITH_WSREP */
     ha_rollback_trans(thd, FALSE);
     if (! thd->in_active_multi_stmt_transaction())
+    {
       thd->tx_isolation= (enum_tx_isolation) thd->variables.tx_isolation;
+      thd->tx_read_only= thd->variables.tx_read_only;
+    }
   }
-
-  RUN_HOOK(transaction, after_rollback, (thd, FALSE));
+  else if (tc_log)
+    tc_log->rollback(thd, false);
 
   thd->transaction.stmt.reset();
 
@@ -555,6 +654,16 @@ bool trans_rollback_to_savepoint(THD *thd, LEX_STRING name)
   SAVEPOINT *sv= *find_savepoint(thd, name);
   DBUG_ENTER("trans_rollback_to_savepoint");
 
+#ifndef DBUG_OFF
+  char buf1[256], buf2[256];
+  DBUG_PRINT("enter", ("stmt.ha_list: %s, all.ha_list: %s",
+                       ha_list_names(thd->transaction.stmt.ha_list, buf1),
+                       ha_list_names(thd->transaction.all.ha_list, buf2)));
+
+  thd->transaction.stmt.dbug_unsafe_rollback_flags("stmt");
+  thd->transaction.all.dbug_unsafe_rollback_flags("all");
+#endif
+
   if (sv == NULL)
   {
     my_error(ER_SP_DOES_NOT_EXIST, MYF(0), "SAVEPOINT", name.str);
@@ -562,20 +671,49 @@ bool trans_rollback_to_savepoint(THD *thd, LEX_STRING name)
   }
 
   enum xa_states xa_state= thd->transaction.xid_state.xa_state;
-  if (xa_state != XA_NOTR)
+  if (xa_state != XA_NOTR && xa_state != XA_ACTIVE)
   {
     my_error(ER_XAER_RMFAIL, MYF(0), xa_state_names[xa_state]);
     DBUG_RETURN(TRUE);
   }
 
+  /**
+    Checking whether it is safe to release metadata locks acquired after
+    savepoint, if rollback to savepoint is successful.
+  
+    Whether it is safe to release MDL after rollback to savepoint depends
+    on storage engines participating in transaction:
+  
+    - InnoDB doesn't release any row-locks on rollback to savepoint so it
+      is probably a bad idea to release MDL as well.
+    - Binary log implementation in some cases (e.g when non-transactional
+      tables involved) may choose not to remove events added after savepoint
+      from transactional cache, but instead will write them to binary
+      log accompanied with ROLLBACK TO SAVEPOINT statement. Since the real
+      write happens at the end of transaction releasing MDL on tables
+      mentioned in these events (i.e. acquired after savepoint and before
+      rollback ot it) can break replication, as concurrent DROP TABLES
+      statements will be able to drop these tables before events will get
+      into binary log,
+  
+    For backward-compatibility reasons we always release MDL if binary
+    logging is off.
+  */
+#ifdef WITH_WSREP
+  bool mdl_can_safely_rollback_to_savepoint=
+                (!((WSREP_EMULATE_BINLOG(thd) ||  mysql_bin_log.is_open()) 
+		   && thd->variables.sql_log_bin) ||
+                 ha_rollback_to_savepoint_can_release_mdl(thd));
+#else
+  bool mdl_can_safely_rollback_to_savepoint=
+                (!(mysql_bin_log.is_open() && thd->variables.sql_log_bin) ||
+                 ha_rollback_to_savepoint_can_release_mdl(thd));
+#endif /* WITH_WSREP */
+
   if (ha_rollback_to_savepoint(thd, sv))
     res= TRUE;
-  else if (((thd->variables.option_bits & OPTION_KEEP_LOG) ||
-            thd->transaction.all.modified_non_trans_table) &&
-           !thd->slave_thread)
-    push_warning(thd, MYSQL_ERROR::WARN_LEVEL_WARN,
-                 ER_WARNING_NOT_COMPLETE_ROLLBACK,
-                 ER(ER_WARNING_NOT_COMPLETE_ROLLBACK));
+  else if (thd->transaction.all.cannot_safely_rollback() && !thd->slave_thread)
+    thd->transaction.push_unsafe_rollback_warnings(thd);
 
   thd->transaction.savepoints= sv;
 
@@ -585,11 +723,10 @@ bool trans_rollback_to_savepoint(THD *thd, LEX_STRING name)
     replication as it allows other connections to drop these tables before
     rollback to savepoint is written to the binlog.
   */
-  bool binlog_on= mysql_bin_log.is_open() && thd->variables.sql_log_bin;
-  if (!res && !binlog_on)
+  if (!res && mdl_can_safely_rollback_to_savepoint)
     thd->mdl_context.rollback_to_savepoint(sv->mdl_savepoint);
 
-  DBUG_RETURN(test(res));
+  DBUG_RETURN(MY_TEST(res));
 }
 
 
@@ -619,12 +756,19 @@ bool trans_release_savepoint(THD *thd, LEX_STRING name)
     DBUG_RETURN(TRUE);
   }
 
+  enum xa_states xa_state= thd->transaction.xid_state.xa_state;
+  if (xa_state != XA_NOTR && xa_state != XA_ACTIVE)
+  {
+    my_error(ER_XAER_RMFAIL, MYF(0), xa_state_names[xa_state]);
+    DBUG_RETURN(TRUE);
+  }
+
   if (ha_release_savepoint(thd, sv))
     res= TRUE;
 
   thd->transaction.savepoints= sv->prev;
 
-  DBUG_RETURN(test(res));
+  DBUG_RETURN(MY_TEST(res));
 }
 
 
@@ -774,7 +918,7 @@ bool trans_xa_commit(THD *thd)
     else
     {
       res= xa_trans_rolled_back(xs);
-      ha_commit_or_rollback_by_xid(thd->lex->xid, !res);
+      ha_commit_or_rollback_by_xid(thd, thd->lex->xid, !res);
       xid_cache_delete(xs);
     }
     DBUG_RETURN(res);
@@ -791,7 +935,7 @@ bool trans_xa_commit(THD *thd)
     wsrep_register_hton(thd, TRUE);
 #endif /* WITH_WSREP */
     int r= ha_commit_trans(thd, TRUE);
-    if ((res= test(r)))
+    if ((res= MY_TEST(r)))
       my_error(r == 1 ? ER_XA_RBROLLBACK : ER_XAER_RMERR, MYF(0));
 #ifdef WITH_WSREP
     wsrep_post_commit(thd, TRUE);
@@ -824,7 +968,11 @@ bool trans_xa_commit(THD *thd)
     {
       DEBUG_SYNC(thd, "trans_xa_commit_after_acquire_commit_lock");
 
-      res= test(ha_commit_one_phase(thd, 1));
+      if (tc_log)
+        res= MY_TEST(tc_log->commit(thd, /* all */ true));
+      else
+        res= MY_TEST(ha_commit_low(thd, /* all */ true));
+
       if (res)
         my_error(ER_XAER_RMERR, MYF(0));
     }
@@ -835,9 +983,11 @@ bool trans_xa_commit(THD *thd)
     DBUG_RETURN(TRUE);
   }
 
-  thd->variables.option_bits&= ~(OPTION_BEGIN | OPTION_KEEP_LOG);
-  thd->transaction.all.modified_non_trans_table= FALSE;
-  thd->server_status&= ~SERVER_STATUS_IN_TRANS;
+  thd->variables.option_bits&= ~OPTION_BEGIN;
+  thd->transaction.all.reset_unsafe_rollback_flags();
+  thd->server_status&=
+    ~(SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
+  DBUG_PRINT("info", ("clearing SERVER_STATUS_IN_TRANS"));
   xid_cache_delete(&thd->transaction.xid_state);
   thd->transaction.xid_state.xa_state= XA_NOTR;
 
@@ -868,10 +1018,10 @@ bool trans_xa_rollback(THD *thd)
     else
     {
       xa_trans_rolled_back(xs);
-      ha_commit_or_rollback_by_xid(thd->lex->xid, 0);
+      ha_commit_or_rollback_by_xid(thd, thd->lex->xid, 0);
       xid_cache_delete(xs);
     }
-    DBUG_RETURN(thd->stmt_da->is_error());
+    DBUG_RETURN(thd->get_stmt_da()->is_error());
   }
 
   if (xa_state != XA_IDLE && xa_state != XA_PREPARED && xa_state != XA_ROLLBACK_ONLY)
@@ -882,9 +1032,11 @@ bool trans_xa_rollback(THD *thd)
 
   res= xa_trans_force_rollback(thd);
 
-  thd->variables.option_bits&= ~(OPTION_BEGIN | OPTION_KEEP_LOG);
-  thd->transaction.all.modified_non_trans_table= FALSE;
-  thd->server_status&= ~SERVER_STATUS_IN_TRANS;
+  thd->variables.option_bits&= ~OPTION_BEGIN;
+  thd->transaction.all.reset_unsafe_rollback_flags();
+  thd->server_status&=
+    ~(SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
+  DBUG_PRINT("info", ("clearing SERVER_STATUS_IN_TRANS"));
   xid_cache_delete(&thd->transaction.xid_state);
   thd->transaction.xid_state.xa_state= XA_NOTR;
 
