@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 #include <mysqld.h>
 #include <sql_class.h>
@@ -26,11 +26,13 @@
 #include <cstdio>
 #include <cstdlib>
 #include "log_event.h"
-#include <slave.h>
+#include <rpl_slave.h>
 
-Format_description_log_event *wsrep_format_desc = NULL;
 wsrep_t *wsrep                  = NULL;
 my_bool wsrep_emulate_bin_log   = FALSE; // activating parts of binlog interface
+/* Sidno in global_sid_map corresponding to group uuid */
+rpl_sidno wsrep_sidno= -1;
+my_bool wsrep_preordered_opt= FALSE;
 
 /*
  * Begin configuration options and their default values
@@ -71,10 +73,6 @@ my_bool wsrep_slave_FK_checks          = 0; // slave thread does FK checks
  */
 
 static const wsrep_uuid_t cluster_uuid = WSREP_UUID_UNDEFINED;
-const wsrep_uuid_t* wsrep_cluster_uuid()
-{
-  return &cluster_uuid;
-}
 static char         cluster_uuid_str[40]= { 0, };
 static const char*  cluster_status_str[WSREP_VIEW_MAX] =
 {
@@ -196,6 +194,23 @@ void wsrep_get_SE_checkpoint(XID* xid)
   plugin_foreach(NULL, get_SE_checkpoint, MYSQL_STORAGE_ENGINE_PLUGIN, xid);
 }
 
+void wsrep_init_sidno(const wsrep_uuid_t& uuid)
+{
+  /* generate new Sid map entry from inverted uuid */
+  rpl_sid sid;
+  wsrep_uuid_t ltid_uuid;
+  for (size_t i= 0; i < sizeof(ltid_uuid.data); ++i)
+  {
+      ltid_uuid.data[i] = ~local_uuid.data[i];
+  }
+  sid.copy_from(ltid_uuid.data);
+  global_sid_lock->wrlock();
+  wsrep_sidno= global_sid_map->add_sid(sid);
+  WSREP_INFO("inited wsrep sidno %d", wsrep_sidno);
+  global_sid_lock->unlock();
+}
+
+
 static wsrep_cb_status_t
 wsrep_view_handler_cb (void*                    app_ctx,
                        void*                    recv_ctx,
@@ -334,6 +349,7 @@ wsrep_view_handler_cb (void*                    app_ctx,
       wsrep_xid_init(&xid, &local_uuid, local_seqno);
       wsrep_set_SE_checkpoint(&xid);
       new_status= WSREP_MEMBER_JOINED;
+      wsrep_init_sidno(local_uuid);
     }
 
     // just some sanity check
@@ -429,8 +445,6 @@ static void wsrep_synced_cb(void* app_ctx)
     if ((rcode = start_slave_threads(1 /* need mutex */,
                             0 /* no wait for start*/,
                             active_mi,
-                            master_info_file,
-                            relay_log_info_file,
                        	    SLAVE_SQL)))
     {
       WSREP_WARN("Failed to create slave threads: %d", rcode);
@@ -482,15 +496,13 @@ static void wsrep_init_position()
   }
 }
 
-extern const char* my_bind_addr_str;
-
 int wsrep_init()
 {
   int rcode= -1;
 
   wsrep_ready_set(FALSE);
   assert(wsrep_provider);
-  wsrep_format_desc= new Format_description_log_event(4);
+
   wsrep_init_position();
 
   if ((rcode= wsrep_load(wsrep_provider, &wsrep, wsrep_log_cb)) != WSREP_OK)
@@ -679,7 +691,7 @@ int wsrep_init()
   return rcode;
 }
 
-extern "C" int wsrep_on(void *);
+extern int wsrep_on(void *);
 
 void wsrep_init_startup (bool first)
 {
@@ -709,9 +721,6 @@ void wsrep_deinit()
   provider_name[0]=    '\0';
   provider_version[0]= '\0';
   provider_vendor[0]=  '\0';
-
-  delete wsrep_format_desc;
-  wsrep_format_desc= NULL;
 }
 
 void wsrep_recover()
@@ -993,9 +1002,15 @@ static bool wsrep_prepare_keys_for_isolation(THD*              thd,
     if (db || table)
     {
         TABLE_LIST tmp_table;
-        bzero((char*) &tmp_table,sizeof(tmp_table));
-        tmp_table.table_name= (char*)db;
-        tmp_table.db= (char*)table;
+	MDL_request mdl_request;
+
+        memset(&tmp_table, 0, sizeof(tmp_table));
+        tmp_table.table_name= (char*)table;
+        tmp_table.db= (char*)db;
+	tmp_table.mdl_request.init(MDL_key::GLOBAL, (db) ? db :  "",
+				   (table) ? table : "",
+				   MDL_INTENTION_EXCLUSIVE, MDL_STATEMENT);
+
         if (!table || !find_temporary_table(thd, &tmp_table))
         {
             if (!(ka->keys= (wsrep_key_t*)my_malloc(sizeof(wsrep_key_t), MYF(0))))
@@ -1118,23 +1133,28 @@ int wsrep_to_buf_helper(
   if (open_cached_file(&tmp_io_cache, mysql_tmpdir, TEMP_PREFIX,
                        65536, MYF(MY_WME)))
     return 1;
-
   int ret(0);
+
+  if (thd->variables.gtid_next.type == GTID_GROUP)
+  {
+      Gtid_log_event gtid_ev(thd, FALSE, &thd->variables.gtid_next);
+      if (!gtid_ev.is_valid()) ret= 0;
+      if (!ret && gtid_ev.write(&tmp_io_cache)) ret= 1;
+  }
+
   /* if there is prepare query, add event for it */
-  if (thd->wsrep_TOI_pre_query)
+  if (!ret && thd->wsrep_TOI_pre_query)
   {
     Query_log_event ev(thd, thd->wsrep_TOI_pre_query,
-                       thd->wsrep_TOI_pre_query_len,
-                       FALSE, FALSE, FALSE, 0);
+		       thd->wsrep_TOI_pre_query_len,
+		       FALSE, FALSE, FALSE, 0);
     if (ev.write(&tmp_io_cache)) ret= 1;
   }
 
-  /* append the actual query */
+  /* continue to append the actual query */
   Query_log_event ev(thd, query, query_len, FALSE, FALSE, FALSE, 0);
-  if (ev.write(&tmp_io_cache)) ret= 1;
-
+  if (!ret && ev.write(&tmp_io_cache)) ret= 1;
   if (!ret && wsrep_write_cache_buf(&tmp_io_cache, buf, buf_len)) ret= 1;
-
   close_cached_file(&tmp_io_cache);
   return ret;
 }
@@ -1376,56 +1396,12 @@ int wsrep_to_isolation_begin(THD *thd, char *db_, char *table_,
                              const TABLE_LIST* table_list)
 {
 
-  LEX *lex;
-  int ret= 0;
   /*
     No isolation for applier or replaying threads.
    */
   if (thd->wsrep_exec_mode == REPL_RECV) return 0;
 
-  lex= thd->lex;
-  /*
-   * Note: We don't check if enforce_storage_engine succeeds or not
-   * We check only the replication requirements here.
-   */
-  if (!wsrep_replicate_myisam && (!enforce_storage_engine || 
-      (enforce_storage_engine && strcasecmp(enforce_storage_engine,"InnoDB")))  && 
-      lex->sql_command == SQLCOM_CREATE_TABLE)
-  {
-    if (table_ && lex->create_info.db_type && \
-        lex->create_info.db_type->db_type == DB_TYPE_MYISAM)
-    {
-        if (db_)
-        {
-            WSREP_INFO("Cannot replicate MyISAM DDL for %s.%s with wsrep_replicate_myisam OFF", db_, table_);
-        }
-        else 
-        {
-            WSREP_INFO("Cannot replicate MyISAM DDL for %s with wsrep_replicate_myisam OFF", table_);
-        }
-        return ret;
-    } 
-    else if (table_list)
-    {
-        for (const TABLE_LIST *table= table_list; table; table= table->next_global)
-        {
-            /*
-             * First condition is required for IF EXISTS queries
-             */
-            if (table->table && table->table->file->ht->db_type == DB_TYPE_MYISAM)
-            {
-                WSREP_INFO("Cannot replicate MyISAM DDL for %s.%s with wsrep_replicate_myisam OFF", table->db, table->table_name);
-                /* 
-                 * Break even if one table is MyISAM
-                 * Note: This may not work well with CREATE TABLE .. SELECT (CTAS)
-                 * but CTAS is not executed as TOI currently.
-                 */
-                return ret;
-            }
-        }
-    }
-  }
-
+  int ret= 0;
   mysql_mutex_lock(&thd->LOCK_wsrep_thd);
 
   if (thd->wsrep_conflict_state == MUST_ABORT)
@@ -1502,10 +1478,10 @@ void wsrep_to_isolation_end(THD *thd)
       msg,                                                                     \
       req->thread_id, (long long)wsrep_thd_trx_seqno(req),                     \
       req->wsrep_exec_mode, req->wsrep_query_state, req->wsrep_conflict_state, \
-      req->command, req->lex->sql_command, req->query(),                       \
+      req->get_command(), req->lex->sql_command, req->query(),		       \
       gra->thread_id, (long long)wsrep_thd_trx_seqno(gra),                     \
       gra->wsrep_exec_mode, gra->wsrep_query_state, gra->wsrep_conflict_state, \
-      gra->command, gra->lex->sql_command, gra->query());
+      gra->get_command(), gra->lex->sql_command, gra->query());
 
 bool
 wsrep_grant_mdl_exception(MDL_context *requestor_ctx,
@@ -1513,8 +1489,8 @@ wsrep_grant_mdl_exception(MDL_context *requestor_ctx,
 ) {
   if (!WSREP_ON) return FALSE;
 
-  THD *request_thd  = requestor_ctx->get_thd();
-  THD *granted_thd  = ticket->get_ctx()->get_thd();
+  THD *request_thd  = requestor_ctx->wsrep_get_thd();
+  THD *granted_thd  = ticket->get_ctx()->wsrep_get_thd();
   bool ret          = FALSE;
 
   mysql_mutex_lock(&request_thd->LOCK_wsrep_thd);
