@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 #ifndef WSREP_MYSQLD_H
 #define WSREP_MYSQLD_H
@@ -19,6 +19,7 @@
 #include "mysqld.h"
 typedef struct st_mysql_show_var SHOW_VAR;
 #include <sql_priv.h>
+#include "rpl_gtid.h"
 #include "../wsrep/wsrep_api.h"
 
 #define WSREP_UNDEFINED_TRX_ID ULONGLONG_MAX
@@ -89,8 +90,8 @@ extern long        wsrep_max_protocol_version;
 extern long        wsrep_protocol_version;
 extern ulong       wsrep_forced_binlog_format;
 extern ulong       wsrep_OSU_method_options;
-extern ulong       wsrep_reject_queries_options;
 extern my_bool     wsrep_desync;
+extern ulong       wsrep_reject_queries_options;
 extern my_bool     wsrep_recovery;
 extern my_bool     wsrep_replicate_myisam;
 extern my_bool     wsrep_log_conflicts;
@@ -147,7 +148,6 @@ extern "C" enum wsrep_conflict_state wsrep_thd_conflict_state(THD *thd);
 extern "C" enum wsrep_query_state wsrep_thd_query_state(THD *thd);
 extern "C" const char * wsrep_thd_exec_mode_str(THD *thd);
 extern "C" const char * wsrep_thd_conflict_state_str(THD *thd);
-extern "C" const char * wsrep_thd_consistency_check_str(THD *thd);
 extern "C" const char * wsrep_thd_query_state_str(THD *thd);
 extern "C" wsrep_ws_handle_t* wsrep_thd_ws_handle(THD *thd);
 
@@ -170,6 +170,7 @@ extern "C" char * wsrep_thd_query(THD *thd);
 extern "C" query_id_t wsrep_thd_wsrep_last_query_id(THD *thd);
 extern "C" void wsrep_thd_set_wsrep_last_query_id(THD *thd, query_id_t id);
 extern "C" void wsrep_thd_awake(THD *thd, my_bool signal);
+extern "C" int wsrep_thd_retry_counter(THD *thd);
 
 
 extern void wsrep_close_client_connections(my_bool wait_to_end);
@@ -194,7 +195,7 @@ extern wsrep_seqno_t wsrep_locked_seqno;
   (global_system_variables.wsrep_on)
 
 #define WSREP(thd) \
-  (WSREP_ON && wsrep && (thd && thd->variables.wsrep_on))
+  (WSREP_ON && (thd && thd->variables.wsrep_on))
 
 #define WSREP_CLIENT(thd) \
     (WSREP(thd) && thd->wsrep_client_thread)
@@ -280,6 +281,8 @@ extern mysql_mutex_t LOCK_wsrep_desync;
 extern wsrep_aborting_thd_t wsrep_aborting_thd;
 extern my_bool       wsrep_emulate_bin_log;
 extern int           wsrep_to_isolation;
+extern rpl_sidno     wsrep_sidno;
+extern my_bool       wsrep_preordered_opt;
 
 #ifdef HAVE_PSI_INTERFACE
 extern PSI_mutex_key key_LOCK_wsrep_ready;
@@ -309,13 +312,13 @@ int wsrep_create_trigger_query(THD *thd, uchar** buf, size_t* buf_len);
 int wsrep_create_event_query(THD *thd, uchar** buf, size_t* buf_len);
 int wsrep_alter_event_query(THD *thd, uchar** buf, size_t* buf_len);
 
-const wsrep_uuid_t* wsrep_cluster_uuid();
 struct xid_t;
+void wsrep_get_SE_checkpoint(xid_t*);
 void wsrep_set_SE_checkpoint(xid_t*);
-
+void wsrep_init_sidno(const wsrep_uuid_t&);
 void wsrep_xid_init(xid_t*, const wsrep_uuid_t*, wsrep_seqno_t);
 const wsrep_uuid_t* wsrep_xid_uuid(const xid_t*);
 wsrep_seqno_t wsrep_xid_seqno(const xid_t*);
-extern "C" int wsrep_is_wsrep_xid(const void* xid);
+int wsrep_is_wsrep_xid(const void* xid);
 
 #endif /* WSREP_MYSQLD_H */
