@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 #include "wsrep_var.h"
 
@@ -37,8 +37,8 @@ const  char* wsrep_node_name        = 0;
 const  char* wsrep_node_address     = 0;
 const  char* wsrep_node_incoming_address = 0;
 const  char* wsrep_start_position   = 0;
-ulong  wsrep_reject_queries_options;
 ulong  wsrep_OSU_method_options;
+ulong   wsrep_reject_queries_options;
 
 int wsrep_init_vars()
 {
@@ -75,13 +75,6 @@ bool wsrep_causal_reads_update (sys_var *self, THD* thd, enum_var_type var_type)
   } else {
     thd->variables.wsrep_sync_wait &= ~WSREP_SYNC_WAIT_BEFORE_READ;
   }
-
-  // update global settings too.
-  if (global_system_variables.wsrep_causal_reads) {
-      global_system_variables.wsrep_sync_wait |= WSREP_SYNC_WAIT_BEFORE_READ;
-  } else {
-      global_system_variables.wsrep_sync_wait &= ~WSREP_SYNC_WAIT_BEFORE_READ;
-  }
   return false;
 }
 
@@ -93,10 +86,6 @@ bool wsrep_sync_wait_update (sys_var* self, THD* thd, enum_var_type var_type)
   // }
   thd->variables.wsrep_causal_reads = thd->variables.wsrep_sync_wait &
           WSREP_SYNC_WAIT_BEFORE_READ;
-
-  // update global settings too
-  global_system_variables.wsrep_causal_reads = global_system_variables.wsrep_sync_wait &
-          WSREP_SYNC_WAIT_BEFORE_READ;
   return false;
 }
 
@@ -219,7 +208,7 @@ static int wsrep_provider_verify (const char* provider_str)
     return 1;
 
   /* check that provider file exists */
-  bzero(&f_stat, sizeof(MY_STAT));
+  memset(&f_stat, 0, sizeof(MY_STAT));
   if (!my_stat(path, &f_stat, MYF(0)))
   {
     return 1;
@@ -506,7 +495,7 @@ bool wsrep_slave_threads_check (sys_var *self, THD* thd, set_var* var)
 {
   if (wsrep_slave_count_change != 0) {
       WSREP_ERROR("Still closing existing threads - %d", abs(wsrep_slave_count_change));
-     push_warning_printf(thd, MYSQL_ERROR::WARN_LEVEL_WARN,
+     push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
                         ER_UNKNOWN_ERROR,
                         "Still closing existing threads - %d",
                         abs(wsrep_slave_count_change));
@@ -535,14 +524,14 @@ bool wsrep_desync_check (sys_var *self, THD* thd, set_var* var)
   bool new_wsrep_desync = var->save_result.ulonglong_value; 
   if (wsrep_desync == new_wsrep_desync) {
     if (new_wsrep_desync) {
-      WSREP_DEBUG("wsrep_desync is already ON.");  
-      push_warning (thd, MYSQL_ERROR::WARN_LEVEL_WARN,
+      WSREP_ERROR("wsrep_desync is already ON.");  
+      push_warning (thd, Sql_condition::WARN_LEVEL_WARN,
                    ER_WRONG_VALUE_FOR_VAR,
                    "'wsrep_desync' is already ON.");
       return true;
     } else {
-      WSREP_DEBUG("wsrep_desync is already OFF.");  
-      push_warning (thd, MYSQL_ERROR::WARN_LEVEL_WARN,
+      WSREP_ERROR("wsrep_desync is already OFF.");  
+      push_warning (thd, Sql_condition::WARN_LEVEL_WARN,
                    ER_WRONG_VALUE_FOR_VAR,
                    "'wsrep_desync' is already OFF.");
       return true;
